@@ -20,7 +20,6 @@ from .base import get_data_home, _fetch_remote, RemoteFileMetadata
 from ..utils import deprecated
 from ..utils import Bunch
 from ..utils._joblib import Memory
-from ..externals.six import b
 from ..utils import _joblib
 
 logger = logging.getLogger(__name__)
@@ -369,7 +368,7 @@ def _fetch_lfw_pairs(index_file_path, data_folder_path, slice_=None,
     # parse the index file to find the number of pairs to be able to allocate
     # the right amount of memory before starting to decode the jpeg files
     with open(index_file_path, 'rb') as index_file:
-        split_lines = [ln.strip().split(b('\t')) for ln in index_file]
+        split_lines = [ln.decode().strip().split('\t') for ln in index_file]
     pair_specs = [sl for sl in split_lines if len(sl) > 2]
     n_pairs = len(pair_specs)
 
