@@ -4,7 +4,6 @@
 import numpy as np
 
 from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import assert_true
 from sklearn.utils.testing import assert_equal
 from sklearn.utils.testing import assert_array_equal
 from sklearn.utils.testing import assert_array_almost_equal
@@ -19,7 +18,7 @@ from sklearn.linear_model import (orthogonal_mp, orthogonal_mp_gram,
 from sklearn.utils import check_random_state
 from sklearn.datasets import make_sparse_coded_signal
 
-n_samples, n_features, n_nonzero_coefs, n_targets = 20, 30, 5, 3
+n_samples, n_features, n_nonzero_coefs, n_targets = 25, 35, 5, 3
 y, X, gamma = make_sparse_coded_signal(n_targets, n_features, n_samples,
                                        n_nonzero_coefs, random_state=0)
 # Make X not of norm 1 for testing
@@ -45,18 +44,18 @@ def test_correct_shapes_gram():
 
 
 def test_n_nonzero_coefs():
-    assert_true(np.count_nonzero(orthogonal_mp(X, y[:, 0],
-                                 n_nonzero_coefs=5)) <= 5)
-    assert_true(np.count_nonzero(orthogonal_mp(X, y[:, 0], n_nonzero_coefs=5,
-                                               precompute=True)) <= 5)
+    assert np.count_nonzero(orthogonal_mp(X, y[:, 0], n_nonzero_coefs=5)) <= 5
+    assert np.count_nonzero(orthogonal_mp(X, y[:, 0],
+                                          n_nonzero_coefs=5,
+                                          precompute=True)) <= 5
 
 
 def test_tol():
     tol = 0.5
     gamma = orthogonal_mp(X, y[:, 0], tol=tol)
     gamma_gram = orthogonal_mp(X, y[:, 0], tol=tol, precompute=True)
-    assert_true(np.sum((y[:, 0] - np.dot(X, gamma)) ** 2) <= tol)
-    assert_true(np.sum((y[:, 0] - np.dot(X, gamma_gram)) ** 2) <= tol)
+    assert np.sum((y[:, 0] - np.dot(X, gamma)) ** 2) <= tol
+    assert np.sum((y[:, 0] - np.dot(X, gamma_gram)) ** 2) <= tol
 
 
 def test_with_without_gram():
@@ -123,12 +122,12 @@ def test_estimator():
     omp.fit(X, y[:, 0])
     assert_equal(omp.coef_.shape, (n_features,))
     assert_equal(omp.intercept_.shape, ())
-    assert_true(np.count_nonzero(omp.coef_) <= n_nonzero_coefs)
+    assert np.count_nonzero(omp.coef_) <= n_nonzero_coefs
 
     omp.fit(X, y)
     assert_equal(omp.coef_.shape, (n_targets, n_features))
     assert_equal(omp.intercept_.shape, (n_targets,))
-    assert_true(np.count_nonzero(omp.coef_) <= n_targets * n_nonzero_coefs)
+    assert np.count_nonzero(omp.coef_) <= n_targets * n_nonzero_coefs
 
     coef_normalized = omp.coef_[0].copy()
     omp.set_params(fit_intercept=True, normalize=False)
@@ -137,14 +136,14 @@ def test_estimator():
 
     omp.set_params(fit_intercept=False, normalize=False)
     omp.fit(X, y[:, 0])
-    assert_true(np.count_nonzero(omp.coef_) <= n_nonzero_coefs)
+    assert np.count_nonzero(omp.coef_) <= n_nonzero_coefs
     assert_equal(omp.coef_.shape, (n_features,))
     assert_equal(omp.intercept_, 0)
 
     omp.fit(X, y)
     assert_equal(omp.coef_.shape, (n_targets, n_features))
     assert_equal(omp.intercept_, 0)
-    assert_true(np.count_nonzero(omp.coef_) <= n_targets * n_nonzero_coefs)
+    assert np.count_nonzero(omp.coef_) <= n_targets * n_nonzero_coefs
 
 
 def test_identical_regressors():
