@@ -1,13 +1,10 @@
 import os
-from os.path import join
-import warnings
 
 from sklearn._build_utils import maybe_cythonize_extensions
 
 
 def configuration(parent_package='', top_path=None):
     from numpy.distutils.misc_util import Configuration
-    from numpy.distutils.system_info import get_info, BlasNotFoundError
     import numpy
 
     libraries = []
@@ -22,6 +19,8 @@ def configuration(parent_package='', top_path=None):
 
     # submodules which do not have their own setup.py
     # we must manually add sub-submodules & tests
+    config.add_subpackage('compose')
+    config.add_subpackage('compose/tests')
     config.add_subpackage('covariance')
     config.add_subpackage('covariance/tests')
     config.add_subpackage('cross_decomposition')
@@ -30,6 +29,10 @@ def configuration(parent_package='', top_path=None):
     config.add_subpackage('feature_selection/tests')
     config.add_subpackage('gaussian_process')
     config.add_subpackage('gaussian_process/tests')
+    config.add_subpackage('impute')
+    config.add_subpackage('impute/tests')
+    config.add_subpackage('inspection')
+    config.add_subpackage('inspection/tests')
     config.add_subpackage('mixture')
     config.add_subpackage('mixture/tests')
     config.add_subpackage('model_selection')
@@ -40,9 +43,12 @@ def configuration(parent_package='', top_path=None):
     config.add_subpackage('preprocessing/tests')
     config.add_subpackage('semi_supervised')
     config.add_subpackage('semi_supervised/tests')
+    config.add_subpackage('experimental')
+    config.add_subpackage('experimental/tests')
+    config.add_subpackage('ensemble/_hist_gradient_boosting')
+    config.add_subpackage('ensemble/_hist_gradient_boosting/tests')
 
     # submodules which have their own setup.py
-    # leave out "linear_model" and "utils" for now; add them after cblas below
     config.add_subpackage('cluster')
     config.add_subpackage('datasets')
     config.add_subpackage('decomposition')
@@ -51,10 +57,11 @@ def configuration(parent_package='', top_path=None):
     config.add_subpackage('feature_extraction')
     config.add_subpackage('manifold')
     config.add_subpackage('metrics')
-    config.add_subpackage('metrics/cluster')
     config.add_subpackage('neighbors')
     config.add_subpackage('tree')
+    config.add_subpackage('utils')
     config.add_subpackage('svm')
+    config.add_subpackage('linear_model')
 
     # add cython extension module for isotonic regression
     config.add_extension('_isotonic',
@@ -63,19 +70,6 @@ def configuration(parent_package='', top_path=None):
                          libraries=libraries,
                          )
 
-    # some libs needs cblas, fortran-compiled BLAS will not be sufficient
-    blas_info = get_info('blas_opt', 0)
-    if (not blas_info) or (
-            ('NO_ATLAS_INFO', 1) in blas_info.get('define_macros', [])):
-        config.add_library('cblas',
-                           sources=[join('src', 'cblas', '*.c')])
-        warnings.warn(BlasNotFoundError.__doc__)
-
-    # the following packages depend on cblas, so they have to be build
-    # after the above.
-    config.add_subpackage('linear_model')
-    config.add_subpackage('utils')
-
     # add the test directory
     config.add_subpackage('tests')
 
@@ -83,6 +77,7 @@ def configuration(parent_package='', top_path=None):
 
     return config
 
+
 if __name__ == '__main__':
     from numpy.distutils.core import setup
     setup(**configuration(top_path='').todict())
